<?php
/**
 * Developped for Stip by Yolknet Internet Services
 * @author Dirk Kokx (dirk@yolknet.nl)
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

require_once JPATH_LIBRARIES. '/easylib/vendor/autoload.php';

// Load the language files
$app = JFactory::getApplication();
$app->getLanguage()->load('lib_easylib', JPATH_ROOT);

/**
 * Class StipEasyImage
 */
class StipEasyImage {

    protected $filename;
    protected $image;
    protected $path = false;
    protected $type;

    /**
     * StipEasyImage constructor.
     * @param JFile $file
     * @param string $type - the name of the image for feedback to the user
     * @param string $extension to convert the image file to (allowed: jpg, png, gif, tif, bmp, ico, psd, webp, data-url)
     * @throws Exception
     */
    public function __construct($file, $type, $extension = 'jpg')
    {

        if (extension_loaded('imagick')){
            $manager = new \Intervention\Image\ImageManager(array('driver' => 'imagick'));
        } else {
            $manager = new \Intervention\Image\ImageManager(array('driver' => 'gd'));
        }

        if(! $manager)
            throw new Exception('Manager kon niet worden geladen');

        if($file){
            try {
                $this->image = $manager->make($file['tmp_name']);
            } catch (Exception $e){
                return [
                    'success' => false,
                    'error' => JText::sprintf("LIB_EASYLIB_EASYIMAGE_ERR_OPENING_IMAGE_FAILED", $type),
                    'type' => $type
                ];
            }
        } else {
            return [
                'success' => false,
                'error' => JText::sprintf("LIB_EASYLIB_EASYIMAGE_ERR_NO_IMAGE_FOUND", $type),
                'type' => $type
            ];
        }

        $this->type = $type;
        $this->extension = $extension;
    }

    /**
     * checks the filename to have the given extension and adapts the name with a postfix (-1, -2 etc) if the file already exists
     * @param $name
     */
    private function setFilename($name)
    {
        $removables = ['\\', '/', ':', '"', '^', '*', '|', '?', '<', '>', ' '];
        $name = str_replace($removables, '-', $name);

        $fragments = explode('.', $name);
        array_pop($fragments);
        $this->filename = implode('.', $fragments).'.'.$this->extension;
        $body = array_pop($fragments);

        $i = 1;
        while(file_exists(JPATH_ROOT.$this->path.$this->filename)){

            if(substr($body, -2, 1) == '-'){
                $body = substr($body, 0, -1).$i;
            } else {
                $body .= '-'.$i;
            }

            if(count($fragments) > 0){
                $this->filename = implode('.', $fragments).'.'.$body.'.'.$this->extension;
            } else {
                $this->filename = $body.'.'.$this->extension;
            }
            $i++;
        }

    }

    /**
     * checks if path exists and if not creates it
     */
    private function checkPath($path)
    {
        if(!$path || $path == ''){
            $path = '/images/';
        } else {

            //fill date strings if available
            $path = $this->fillDates($path);

            //add slash at the beginning because it's not defined in the base path
            if(substr($path, 0, 1) != '/')
                $path = '/'.$path;

            //check slash at the end, remove it as well
            if(substr($path, -1) != '/')
                $path = $path.'/';

            if(substr($path, 0, 7) !== '/images')
                $path = '/images'.$path;

            $dirs = explode('/', $path);

            array_pop($dirs);
            $fullpath = JPATH_ROOT.'/'.$dirs[1];

            if(count($dirs) > 2){
                //create dirs if not yet set
                $i = 2;
                while($i < count($dirs)){
                    $fullpath .= '/'.$dirs[$i];
                    if(! is_dir($fullpath)){
                        mkdir($fullpath, 0777);
                    }
                    $i++;
                }
            }
        }

        $this->path = $path;
        return $path;
    }

    private function fillDates($path)
    {
        $result = '';
        $parts = preg_split( "/ (\{|\}) /", $path );
        foreach ($parts as $part) {
            if(substr($part, 0, 5) === 'month' || substr($part, 0, 4) === 'year') {
                $dateSegments = explode(' ',$part);
                $locale = 'en_GB';
                $format = 'default';
                foreach ($dateSegments as $dateSegment) {
                    switch ($dateSegment) {
                        case 'month':
                            $type = 'month';
                            break;
                        case 'year':
                            $type = 'year';
                            break;
                        case substr($dateSegment, 0, 6) === 'format':
                            $data = explode('=', $dateSegment);
                            $format = $data[1];
                            break;
                        case substr($dateSegment, 0, 6) === 'locale':
                            $data = explode('=', $dateSegment);
                            $locale = $data[1];
                            break;
                        default:
                            break;
                    }
                }
                if(strlen($locale) !== 5) {
                    $locale = 'en_GB';
                } else if ($locale !== 'en_GB') {
                    $locs = explode('_', $locale);
                    if(count($locs) !== 2) {
                        $locale = 'en_GB';
                    } else {
                        $locale = strtolower($locs[0]).'_'.strtoupper($locs[1]);
                    }
                }

                if($type === 'month') {
                    switch($format) {
                        case 'n':
                        case 'm':
                            $result .= date("m");
                            break;
                        case 'F':
                            setlocale(LC_TIME, $locale);
                            $result .= strftime('%B');
                            break;
                        case 'M':
                            setlocale(LC_TIME, $locale);
                            $result .= strftime('%b');
                            break;
                    }
                } else {
                    switch($format) {
                        case 'Y':
                        case 'y':
                            $result .= date($format);
                            break;
                        default:
                            $result .= date('Y');
                            break;
                    }
                }
            } else {
                $result .= $part;
            }
        }
        return $result;
    }


    /**
     * @param (int or null) $width
     * @param (int or null) $height
     * @param string $name
     * @param mixed $path
     * @param int $quality
     * @return array
     */
    public function saveImage($width, $height, $name, $path = false, $quality = 90)
    {
        //cleaning input
        $quality = (int) $quality;
        $width = ($width == 0)? null : $width;
        $height = ($height == 0)? null : $height;

        //set path and filename
        $this->checkPath($path);
        $this->setFilename($name);

        if($width && $height){
            //resize en crop het bestand
            $this->image->fit($width, $height);
        } else {
            //resize het bestand
            $this->image->resize($width, $height, function ($constraint) {
                $constraint->aspectRatio();
            });
        }

        //sharpen, depending on the amount of minification
        if($width < 0.9 * $this->image->getWidth() || $height < 0.9 * $this->image->getHeight() ){
            $baseamount = 5;
            if($width){
                $amount =  (int) $this->image->getWidth() / $width;
                if($amount > $baseamount)
                    $baseamount = $amount;
            }

            if($height){
                $amount =  (int) $this->image->getHeight() / $height;
                if($amount > $baseamount)
                    $baseamount = $amount;
            }

            $amount = ($baseamount > 100)? 100 : $baseamount;
            $this->image->sharpen($amount);
        }

        //we only need to set quality if the image concerns a jpeg image
        if($this->extension == 'jpg')
            $this->image->encode('jpg', $quality);

        //try to store the file on the given location and give feedback on the result
        $file = JPATH_ROOT.$this->path.$this->filename;
        try{
            $this->image->save($file);
            return [
                'success' => true,
                'filename' => substr($this->path.$this->filename, 1)
            ];
        } catch (Exception $e){
            return [
                'succes' => false,
                'error' => JText::sprintf("LIB_EASYLIB_EASYIMAGE_ERR_SAVING_IMAGE_FAILED", $this->type, $e)
            ];
        }
    }

}
