<?php
/**
 * Developped for Stip by Yolknet Internet Services
 * @author Dirk Kokx (dirk@yolknet.nl)
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// Load the language files
$app = JFactory::getApplication();
$app->getLanguage()->load('lib_easylib', JPATH_ROOT);

/**
 * Contains functions to prepare the form to show EasyImage Field
 * Class StipEasyimageFormHelper
 */
class StipEasyimageFormHelper {

    /**
     * Holds the current group the fields belong to
     * @var    string
     */
    private $fieldGroup = '';
    private $fieldPath = false;
    private $ajaxUrl = '';

    /**
     * @param JForm $form
     * @param bool $with_select
     * @param array $change_fields [ fieldname => fielddescription] - field to change to easyimage field
     * @param array $hide_fields [ fieldname => fielddescription] - field to hide
     */
    public function setFormFields(&$form, &$data, $with_select, $ajax_url, $change_fields = [], $hide_fields = [])
    {
        $this->setAjaxPath($ajax_url);

        //change the media fields to stip easy image fields
        foreach ($form->getXml() as $set => $item) {

            // Find the fields for this item
            $fieldCollection = $this->findFields($item);

            // Check to see if we have any fields
            if (!$fieldCollection) {
                continue;
            }

            /** @var SimpleXMLElement $field */
            foreach ($fieldCollection as $groupName => $fields) {

                switch ($fields['path'])
                {
                    case 'fieldset/fields':
                    case 'fieldset':
                        $name = (string) $item->attributes()->name;

                        if (!$this->fieldGroup && $name)
                        {
                            $this->fieldGroup = $name;
                        }

                        if (!$name)
                        {
                            $name = (string) $item->fieldset->attributes()->name;
                        }

                        $identifier[] = $name;
                        break;
                    case 'fields/fieldset':
                    case 'fields':
                        $this->fieldGroup = (string) $item->fields->attributes()->name;
                        $identifier[]     = $this->fieldGroup;
                        break;
                    case 'field':
                        $this->fieldGroup = $groupName;
                        break;
                }

                if(count($change_fields) > 0 || count($hide_fields) > 0) {
                    foreach ($fields['fields'] as $field) {
                        $attributes = $field->attributes();

                        if (count($change_fields) > 0) {
                            if (in_array($attributes->name, array_keys($change_fields))) {

                                $new_attributes = $change_fields["$attributes->name"];
                                foreach ($new_attributes as $name => $value) {
                                    $form->setFieldAttribute(
                                        (string)$attributes->name,
                                        $name,
                                        $value,
                                        $this->fieldGroup
                                    );
                                }

                                //change to easyimage
                                $this->changeToStipImage($form, $attributes);

                                if ($with_select)
                                    $this->addSelectField($form, $data, (string)$groupName, (string)$attributes->name);
                            }
                        }

                        if (count($hide_fields) > 0) {
                            if (in_array($attributes->name, $hide_fields)) {
                                $form->setFieldAttribute(
                                    (string)$attributes->name,
                                    'type',
                                    'hidden',
                                    $this->fieldGroup
                                );
                            }
                        }
                    }
                }

            }
        }

    }

    /**
     * cleans the url path if neccessary
     * @param $path
     * @throws Exception
     */
    private function setAjaxPath($path)
    {
        if(substr($path, 0, 4) == 'http')
        {
            //we only allow paths relative to the current website
            list($pfx, $path) = explode('index.php', $path);
            if(! $path){
                throw new Exception(JText::_('LIB_EASYLIB_EASYIMAGE_ERR_INVALID_AJAXURL_START_WITH_INDEX'));
            } else {

            }
            $this->ajaxUrl = $path;
        } elseif (substr($path, 0, 10) == '/index.php'){
            $this->ajaxUrl = substr($path, 1);
        } elseif (substr($path, 0, 9) == 'index.php') {
            $this->ajaxUrl = $path;
        } else {
            throw new Exception(JText::_('LIB_EASYLIB_EASYIMAGE_ERR_INVALID_AJAXURL'));
        }
    }

    /**
     * additional field that gets triggered by the select button to open the filebrowser
     * @param $form
     * @param $data (object or array)
     * @param $groupName
     * @param $origName
     */
    public function addSelectField(&$form, &$data, $groupName, $origName)
    {
        $form->load('
            <form>
                <fields name="'.$groupName.'">
                    <fieldset>
                        <field
                            name="stip_select_'.$origName.'"
                            type="media"
                            accept="image/*"
                            preview="false"
                            id="stip_select_'.$origName.'"
                            class="stip_select"
                        />
                    </fieldset>
                </fields>
            </form>
        ');

        if(is_array($data)){
            if(isset($data[$groupName])){
                $groupdata = (is_array($data[$groupName]))? $data[$groupName] : json_decode($data[$groupName], 1);
                if($groupdata[$origName])
                    $groupdata["stip_select_".$origName] = $groupdata[$origName];

                $data[$groupName] = (is_array($data[$groupName]))? $groupdata : json_encode($groupdata);
            }
        } else {
            if(isset($data->$groupName)){
                $groupdata = (is_array($data->$groupName))? $data->$groupName : json_decode($data->$groupName, 1);
                if(isset($groupdata[$origName]))
                    $groupdata["stip_select_".$origName] = $groupdata[$origName];

                $data->$groupName = (is_array($data->$groupName))? $groupdata : json_encode($groupdata);
            }
        }
    }

    private function findFields($item)
    {
        $fieldCollection = array();
        $paths  = array(
            'fieldset/fields',
            'fields/fieldset',
            'fieldset',
            'fields',
        );

        // First find the groups
        foreach ($paths as $index => $path)
        {
            $groups = $item->xpath($path);

            if ($groups)
            {
                foreach ($groups as $group)
                {
                    $groupName = (string) $group->attributes()->name;

                    $fieldCollection[$groupName]['path']   = $path;
                    $fieldCollection[$groupName]['fields'] = $group->xpath('field');
                }

                break;
            }
        }

        // Second find the fields
        if (empty($fieldCollection))
        {
            $fields = $item->xpath('field');

            if ($fields)
            {
                $groupName                             = (string) $item->attributes()->name;
                $fieldCollection[$groupName]['path']   = 'field';
                $fieldCollection[$groupName]['fields'] = $fields;
            }
        }

        return $fieldCollection;
    }

    private function changeToStipImage(JForm &$form, $attributes)
    {

        $form->setFieldAttribute(
            (string) $attributes->name,
            'type',
            "easyimage",
            $this->fieldGroup
        );

        $form->setFieldAttribute(
            (string) $attributes->name,
            'ajaxurl',
            $this->ajaxUrl,
            $this->fieldGroup
        );

        $form->setFieldAttribute(
            (string) $attributes->name,
            'accept',
            "image/*",
            $this->fieldGroup
        );

        if(! $this->fieldPath) {
            $form->setFieldAttribute(
                (string) $attributes->name,
                'addfieldpath',
                "libraries/easylib/forms/fields",
                $this->fieldGroup
            );

            $this->fieldPath = true;
        }
    }

    public static function setAssets($custom_js = false)
    {

        if($custom_js){
            $doc = JFactory::getDocument();
            $doc->addScriptDeclaration($custom_js);
        }

    }

    public static function processAjax($quality = 90)
    {
        $jinput = JFactory::getApplication()->input;
        $file = $jinput->files->get('file');
        $filename = $jinput->getString('filename');
        $type = $jinput->get('type');
        $width = $jinput->get('width');
        $height = $jinput->get('height');
        $folder = $jinput->get('folder', null, 'RAW');

        try {
            $easyImage = new StipEasyImage($file, $type);
        } catch (Exception $e){
            return $e;
        }

        return $easyImage->saveImage($width, $height, $filename, $folder, $quality);

    }
}
